//---------------------------------------------------------------------------
// IniFiles.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "IniFiles.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace IniFiles {
//---------------------------------------------------------------------------
__fastcall TVarIniFile::TVarIniFile(const AnsiString& AFileName) :
	TMemIniFile(AFileName)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TVarIniFile::~TVarIniFile()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TRect __fastcall TVarIniFile::ReadRect(const AnsiString& ASection, const AnsiString& AIdent, const TRect& ADefault)
{
	if(!ValueExists(ASection, AIdent)) {
		return ADefault;
	}
	TPtr<TStrings> record = new TStringList();
	try {
		record->CommaText = TMemIniFile::ReadString(ASection, AIdent, AnsiString());
		return TRect(
			record->Strings[0].ToInt(),
			record->Strings[1].ToInt(),
			record->Strings[2].ToInt(),
			record->Strings[3].ToInt());
	}
	catch(EListError& e) {
		throw EIniFileException("The program cannot read a point from a INI file.");
	}
	catch(EConvertError& e) {
		throw EIniFileException("The program cannot read a point from a INI file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WriteRect(const AnsiString& ASection, const AnsiString& AIdent, const TRect& AValue)
{
	TPtr<TStrings> record = new TStringList();
	record->Add(AValue.Left);
	record->Add(AValue.Top);
	record->Add(AValue.Right);
	record->Add(AValue.Bottom);
	TMemIniFile::WriteString(ASection, AIdent, record->CommaText);
}
//---------------------------------------------------------------------------
TPoint __fastcall TVarIniFile::ReadPoint(const AnsiString& ASection, const AnsiString& AIdent, const TPoint& ADefault)
{
	if(!ValueExists(ASection, AIdent)) {
		return ADefault;
	}
	TPtr<TStrings> record = new TStringList();
	try {
		record->CommaText = TMemIniFile::ReadString(ASection, AIdent, AnsiString());
		return TPoint(
			record->Strings[0].ToInt(),
			record->Strings[1].ToInt());
	}
	catch(EListError& e) {
		throw EIniFileException("The program cannot read a point from a INI file.");
	}
	catch(EConvertError& e) {
		throw EIniFileException("The program cannot read a point from a INI file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WritePoint(const AnsiString& ASection, const AnsiString& AIdent, const TPoint& AValue)
{
	TPtr<TStrings> record = new TStringList();
	record->Add(AValue.x);
	record->Add(AValue.y);
	TMemIniFile::WriteString(ASection, AIdent, record->CommaText);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TVarIniFile::ReadQuotedString(const AnsiString& ASection, const AnsiString& AIdent, AnsiString ADefault, char AQuoteChar)
{
	if(!ValueExists(ASection, AIdent)) {
		return ADefault;
	}
	AnsiString value = TMemIniFile::ReadString(ASection, AIdent, ADefault);
	if(value.Length() < 2 || value[1] != AQuoteChar || value[value.Length()] != AQuoteChar) {
		throw EIniFileException("The program cannot read a quote string from a INI file.");
	}
	return value.SubString(2, value.Length() - 2);
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WriteQuotedString(const AnsiString& ASection, const AnsiString& AIdent, AnsiString AValue, char AQuoteChar)
{
	TMemIniFile::WriteString(ASection, AIdent, AnsiString().sprintf("%c%s%c", AQuoteChar, AValue.c_str(), AQuoteChar));
}
//---------------------------------------------------------------------------
__int64 __fastcall TVarIniFile::ReadInteger64(const AnsiString& ASection, const AnsiString& AIdent, __int64 ADefault)
{
	if(!ValueExists(ASection, AIdent)) {
		return ADefault;
	}
	try {
		return StrToInt64(TMemIniFile::ReadString(ASection, AIdent, ADefault));
	}
	catch(EConvertError& e) {
		throw EIniFileException("The program cannot read an __int64 value from a INI file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WriteInteger64(const AnsiString& ASection, const AnsiString& AIdent, __int64 AValue)
{
	TMemIniFile::WriteString(ASection, AIdent, AValue);
}
//---------------------------------------------------------------------------
unsigned __int64 __fastcall TVarIniFile::ReadUInteger64(const AnsiString& ASection, const AnsiString& AIdent, unsigned __int64 ADefault)
{
	if(!ValueExists(ASection, AIdent)) {
		return ADefault;
	}
	try {
		return static_cast<unsigned __int64>(StrToInt64(TMemIniFile::ReadString(ASection, AIdent, ADefault)));
	}
	catch(EConvertError& e) {
		throw EIniFileException("The program cannot read an unsigned __int64 value from a INI file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WriteUInteger64(const AnsiString& ASection, const AnsiString& AIdent, unsigned __int64 AValue)
{
	TMemIniFile::WriteString(ASection, AIdent, AValue);
}
//---------------------------------------------------------------------------
unsigned int __fastcall TVarIniFile::ReadUInteger(const AnsiString& ASection, const AnsiString& AIdent, unsigned int ADefault)
{
	return TMemIniFile::ReadInteger(ASection, AIdent, ADefault);
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WriteUInteger(const AnsiString& ASection, const AnsiString& AIdent, unsigned int AValue)
{
	TMemIniFile::ReadInteger(ASection, AIdent, AValue);
}
//---------------------------------------------------------------------------
double __fastcall TVarIniFile::ReadDouble(const AnsiString& ASection, const AnsiString& AIdent, double ADefault)
{
	if(!ValueExists(ASection, AIdent)) {
		return ADefault;
	}
	try {
		return TMemIniFile::ReadString(ASection, AIdent, ADefault).ToDouble();
	}
	catch(EConvertError& e) {
		throw EIniFileException("The program cannot read a double value from a INI file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TVarIniFile::WriteDouble(const AnsiString& ASection, const AnsiString& AIdent, double AValue)
{
	TMemIniFile::WriteString(ASection, AIdent, AValue);
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::IniFiles
//---------------------------------------------------------------------------

