//---------------------------------------------------------------------------
// Taskbar.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Math.hpp>
#include "Taskbar.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Taskbar {
//---------------------------------------------------------------------------
const UINT TNotificationIcon::WM_NOTIFICATIONICON = WM_APP + 0;
const UINT TNotificationIcon::WM_TASKBARCREATED = ::RegisterWindowMessage("TaskbarCreated");
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::Initialize()
{
	try {
		FHandle = AllocateHWnd(WndProc);

		FIcon = new TIcon();
		FIcon->OnChange = IconChange;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::Uninitialize()
{
	delete FIcon;
	FIcon = NULL;

	if(FHandle != NULL) {
		DeallocateHWnd(FHandle);
		FHandle = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::Update(DWORD AKind)
{
	if(AKind == NIM_MODIFY && !Visible) {
		return;
	}
	NOTIFYICONDATA data = {0};
	data.cbSize = sizeof(NOTIFYICONDATA);
	data.hWnd = Handle;
	data.uID = Identifier;
	data.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	data.hIcon = Icon->Handle;
	if(ShowHint) {
		::lstrcpyn(
			data.szTip,
			Hint.c_str(),
			Min(sizeof(data.szTip) / sizeof(char), Hint.Length() + 1));
	}
	data.uCallbackMessage = WM_NOTIFICATIONICON;

	// Do not check the return value. There might be a gap between the actual
	// icon data and the cache.
	::Shell_NotifyIcon(AKind, &data);
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::WndProc(TMessage& AMessage)
{
	if(AMessage.Msg == WM_NOTIFICATIONICON) {
		WMNotificationIcon(AMessage);
	}
	else if(AMessage.Msg == WM_TASKBARCREATED) {
		WMTaskbarCreated(AMessage);
	}
	else {
		AMessage.Result = ::DefWindowProc(Handle, AMessage.Msg, AMessage.WParam, AMessage.LParam);
	}
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::WMNotificationIcon(TMessage& AMessage)
{
	DoMessage(AMessage.LParam);
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::WMTaskbarCreated(TMessage& AMessage)
{
	if(Visible) {
		Visible = false;
		Visible = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::IconChange(TObject* ASender)
{
	Update(NIM_MODIFY);
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::DoMessage(UINT AKind)
{
	if(OnMessage != NULL) {
		OnMessage(this, AKind);
	}
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::SetHint(const AnsiString& AHint)
{
	if(AHint == Hint) {
		return;
	}
	FHint = AHint;
	Update(NIM_MODIFY);
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::SetShowHint(bool AShowHint)
{
	if((AShowHint ? 1 : 0) == (ShowHint ? 1 : 0)) {
		return;
	}
	FShowHint = AShowHint;
	if(Visible) {
		Visible = false;
		Visible = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TNotificationIcon::SetVisible(bool AVisible)
{
	if((AVisible ? 1 : 0) == (Visible ? 1 : 0)) {
		return;
	}
	FVisible = AVisible;
	Update(AVisible ? NIM_ADD : NIM_DELETE);
}
//---------------------------------------------------------------------------
__fastcall TNotificationIcon::TNotificationIcon(int AIdentifier) :
	TObject(),
	FHandle(NULL),
	FIdentifier(AIdentifier),
	FIcon(NULL),
	FHint(),
	FShowHint(false),
	FVisible(false),
	FOnMessage(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TNotificationIcon::~TNotificationIcon()
{
	Visible = false;
	Uninitialize();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Taskbar
//---------------------------------------------------------------------------

