//---------------------------------------------------------------------------
// ScrollSnap.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <windows.h>
#pragma hdrstop
#include "ScrollSnap.h"
#include "WheelCommon.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace WheelHere { namespace Wheeler { namespace ScrollSnap {
//---------------------------------------------------------------------------
void TScrollBarSnap::Update(HWND AWindow, int AKind)
{
	SCROLLINFO scrollInfo = {0};
	scrollInfo.cbSize = sizeof(SCROLLINFO);
	scrollInfo.fMask = SIF_ALL;
	{
		LONG_PTR windowStyle = EnsureGetWindowLongPtr(AWindow, GWL_STYLE);
		switch(AKind) {
		case SB_CTL:
			// Calling GetScrollInfo causes an error on status bars
			if(!::SendMessage(
				AWindow,
				SBM_GETSCROLLINFO,
				0,
				reinterpret_cast<LPARAM>(&scrollInfo))) {
				return;
			}
			break;
		case SB_HORZ:
			if((windowStyle & WS_HSCROLL) == 0 || !::GetScrollInfo(AWindow, SB_HORZ, &scrollInfo)) {
				return;
			}
			break;
		case SB_VERT:
			if((windowStyle & WS_VSCROLL) == 0 || !::GetScrollInfo(AWindow, SB_VERT, &scrollInfo)) {
				return;
			}
			break;
		default:
			throw EWindowError();
		}
	}
	FMin = scrollInfo.nMin;
	FMax = scrollInfo.nMax;
	FPosition = scrollInfo.nPos;
	FPage = scrollInfo.nPage;
	FAvailable = true;
}
//---------------------------------------------------------------------------
TScrollBarSnap::TScrollBarSnap(HWND AWindow, int AKind) :
	FMin(0),
	FMax(0),
	FPosition(0),
	FPage(0),
	FAvailable(false)
{
	Update(AWindow, AKind);
}
//---------------------------------------------------------------------------
TScrollBarSnap::TScrollBarSnap() :
	FMin(0),
	FMax(0),
	FPosition(0),
	FPage(0),
	FAvailable(false)
{
	// Do nothing
}
//---------------------------------------------------------------------------
TScrollBarSnap::~TScrollBarSnap()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool TScrollBarSnap::operator ==(const TScrollBarSnap& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	if(!Available && !AOther.Available) {
		return true;
	}
	return
		Min == AOther.Min &&
		Max == AOther.Max &&
		Position == AOther.Position &&
		Page == AOther.Page &&
		(Available ? 1 : 0) == (AOther.Available ? 1 : 0);
}
//---------------------------------------------------------------------------
bool TScrollBarSnap::operator !=(const TScrollBarSnap& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------
bool TScrollBarSnap::CanScroll() const
{
	return CanScrollTo(true) || CanScrollTo(false);
}
//---------------------------------------------------------------------------
bool TScrollBarSnap::CanScrollTo(bool AForward) const
{
	if(!Available) {
		return false;
	}
	if(Min >= Max || Page >= (Max - Min + 1)) {
		return false;
	}
	if(AForward) {
		return Position <= Max - Page;
	}
	else {
		return Position > Min;
	}
}
//---------------------------------------------------------------------------
TScrollSnap::TScrollSnap(HWND AWindow) :
	FControlSnap(AWindow, SB_CTL),
	FHorzBarSnap(AWindow, SB_HORZ),
	FVertBarSnap(AWindow, SB_VERT),
	FHorzControl(false)
{
	if(ControlSnap.Available) {
		// SBS_VERT is definded as 1 and SBS_HORZ as 0
		if((EnsureGetWindowLongPtr(AWindow, GWL_STYLE) & SBS_VERT) == 0) {
			FHorzControl = true;
		}
	}
}
//---------------------------------------------------------------------------
TScrollSnap::TScrollSnap() :
	FControlSnap(),
	FHorzBarSnap(),
	FVertBarSnap(),
	FHorzControl(false)
{
	// Do nothing
}
//---------------------------------------------------------------------------
TScrollSnap::~TScrollSnap()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool TScrollSnap::operator ==(const TScrollSnap& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	return
		FControlSnap == AOther.FControlSnap &&
		FHorzBarSnap == AOther.FHorzBarSnap &&
		FVertBarSnap == AOther.FVertBarSnap;
}
//---------------------------------------------------------------------------
bool TScrollSnap::operator !=(const TScrollSnap& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------
bool TScrollSnap::CanScroll() const
{
	return
		FControlSnap.CanScroll() ||
		FHorzBarSnap.CanScroll() ||
		FVertBarSnap.CanScroll();
}
//---------------------------------------------------------------------------
}}}} // End of namespace Moochos::WheelHere::Wheeler::ScrollSnap
//---------------------------------------------------------------------------

